@@ -135,14 +135,15 @@ class PlainMechanism(Mechanism):
 
     allows_anonymous = False
 
-    def __init__(self, sasl, username=None, password=None, **props):
+    def __init__(self, sasl, username=None, password=None, identity='', **props):
         Mechanism.__init__(self, sasl)
+        self.identity = identity
         self.username = username
         self.password = password
 
     def process(self, challenge=None):
         self._fetch_properties('username', 'password')
-        return b'\x00' + bytes(self.username) + b'\x00' + bytes(self.password)
+        return bytes(self.identity) + b'\x00' + bytes(self.username) + b'\x00' + bytes(self.password)
 
     def dispose(self):
         self.password = None
@@ -451,8 +452,14 @@ class GSSAPIMechanism(Mechanism):
         self._fetch_properties('host', 'service')
 
         krb_service = b'@'.join((bytes(self.service), bytes(self.host)))
-        _, self.context = kerberos.authGSSClientInit(
-                service=krb_service, principal=self.principal)
+        try:
+            _, self.context = kerberos.authGSSClientInit(
+                    service=krb_service, principal=self.principal)
+        except TypeError:
+            if self.principal is not None:
+                raise StandardError("Error: kerberos library does not support principal.")
+            _, self.context = kerberos.authGSSClientInit(
+                    service=krb_service)
 
     def process(self, challenge=None):
         if not self._have_negotiated_details:
